/**
 * Copyright (C) 2015 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.fabric8.kubernetes.api.model.autoscaling;

import org.junit.jupiter.api.Test;
import io.fabric8.kubernetes.api.model.autoscaling.v2beta2.HorizontalPodAutoscaler;
import io.fabric8.kubernetes.api.model.autoscaling.v2beta2.MetricSpecBuilder;
import io.fabric8.kubernetes.api.model.autoscaling.v2beta2.HorizontalPodAutoscalerBuilder;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class HorizontalPodAutoscalerTest {
  @Test
  public void testBuilder() {
    HorizontalPodAutoscaler horizontalPodAutoscaler = new HorizontalPodAutoscalerBuilder()
      .withNewMetadata().withName("the-hpa").withNamespace("default").endMetadata()
      .withNewSpec()
      .withNewScaleTargetRef()
      .withApiVersion("apps/v1")
      .withKind("Deployment")
      .withName("the-deployment")
      .endScaleTargetRef()
      .withMinReplicas(1)
      .withMaxReplicas(10)
      .addToMetrics(new MetricSpecBuilder()
        .withType("Resource")
        .withNewResource()
        .withName("cpu")
        .withNewTarget()
        .withType("Utilization")
        .withAverageUtilization(50)
        .endTarget()
        .endResource()
        .build())
      .endSpec()
      .build();

    assertEquals("the-hpa", horizontalPodAutoscaler.getMetadata().getName());
    assertEquals(1, horizontalPodAutoscaler.getSpec().getMetrics().size());
  }
}
